package funktiopiirturi;

import javax.swing.*;

import java.awt.Component;
//import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.ArrayList;
import java.util.ListIterator;

/**
 * Funktion syttpaneeli sislt tekstikentn, johon funktion oikea puoli sytetn.
 * Enteri tai "Piirr"-nappia painamalla sytetty funktio muunnetaan piirtopinnan
 * ymmrtmn muotoon ja lhetetn piirrettvksi. 
 *
 */
public class FunctionInputPanel extends JPanel implements ActionListener {
	final static long serialVersionUID = 1L;	// ei varoituksia kiitos..
	private JLabel prompt;						// teksti "y =" syttekentn vasemmalla
	private JTextField inputField;				// funktioiden syttekentt
	private JButton drawButton;					// Piirr-nappi
	private Funktiopiirturi parent;				// viite emo-ohjelmaan mm. virheilm. varten
	
	private static final int MAX_HISTORY_LENGTH = 100;	// aika selv...
	private List<String> history;						// viimeisimmt sytetyt funktiot
	private ListIterator<String> currentHistoryIndex;	// mit kohtaa historiasta selataan
	
	/**
	 * Action-luokka sytthistorian selaamiseen takaisinpin.
	 */
	private class PreviousAction extends AbstractAction {
		final static long serialVersionUID = 1L;
		private FunctionInputPanel parent;
		public PreviousAction(FunctionInputPanel parent) {
			super();
			this.parent = parent;
		}
		public void actionPerformed(ActionEvent e) {
			this.parent.browsePrevious();
		}
	}
	
	/**
	 * Action-luokka sytthistorian selaamiseen eteenpin.
	 */
	private class NextAction extends AbstractAction {
		final static long serialVersionUID = 1L;
		private FunctionInputPanel parent;
		public NextAction(FunctionInputPanel parent) {
			super();
			this.parent = parent;
		}
		public void actionPerformed(ActionEvent e) {
			this.parent.browseNext();
		}
	}
	
	private PreviousAction previousAction;	// Action-oliot, joiden actionPerformed-metodia
	private NextAction nextAction;			// kutsutaan, yls- ja alas-nppimist.
	
	/**
	 * Luo uuden funktionsyttpaneelin, joka vlitt lukemansa funktiot sek
	 * virheilmoitukset parametrina annettavalle parent-oliolle.
	 * @param parent	Funktiopiirturi-olio, jolle kyttjn syttmt funktiot vlitetn.
	 */
	public FunctionInputPanel(Funktiopiirturi parent) {
		this.parent = parent;
		
		// Luodaan action-oliot nppimill selattavaa historiaa varten.
		this.previousAction = new PreviousAction(this);
		this.nextAction = new NextAction(this);
		
		// alustetaan historia
		this.history = new ArrayList<String>();
		this.currentHistoryIndex = history.listIterator(this.history.size());
		
		// syttkehote
		this.prompt = new JLabel(" y = ");
		
		// funktion syttkentt
		this.inputField = new JTextField();
		this.inputField.addActionListener(this);
		
//		javax.swing.text.Keymap k = this.inputField.getKeymap();
//		Action a = k.getAction(KeyStroke.getKeyStroke("^"));
//		System.out.println(a);
		
		// Piirr -nappi
		this.drawButton = new JButton("Piirr");
		this.drawButton.addActionListener(this);
		
		// rakennetaan this loppuun
		this.add(this.prompt);
		this.add(this.inputField);
		this.add(this.drawButton);
		this.setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		this.setAlignmentX(Component.RIGHT_ALIGNMENT);
		
		// yls-nppimen painalluksen ksittelee previousAction-olio
		this.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(
				KeyStroke.getKeyStroke("UP"), "previous");
		this.getActionMap().put("previous", this.previousAction);
		
		// alas-nppimen painalluksen ksittelee nextAction-olio
		this.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(
				KeyStroke.getKeyStroke("DOWN"), "next");
		this.getActionMap().put("next", this.nextAction);
	}
	
	/**
	 * Syttpaneeliin focusoitaessa, siirretn focus syttkenttn.
	 */
	public void requestFocus() {
		//if( !this.hasFocus() )
		//	super.requestFocus();
		if( !this.inputField.hasFocus() )
			this.inputField.requestFocus();
	}
	
	/**
	 * Selaa sytthistoriaa taaksepin (vanhempaan pin).
	 */
	void browsePrevious() {
		if( this.currentHistoryIndex.hasPrevious() ) {
			this.inputField.setText(this.currentHistoryIndex.previous());
			this.inputField.setSelectionStart(0);
			this.inputField.setSelectionEnd(this.inputField.getText().length());
		}
	}
	
	/**
	 * Selaa sytthistoriaa eteenpin (uudempaan pin).
	 */
	void browseNext() {
		if( this.currentHistoryIndex.hasNext() ) {
			this.inputField.setText(this.currentHistoryIndex.next());
			this.inputField.setSelectionStart(0);
			this.inputField.setSelectionEnd(this.inputField.getText().length());
		} else
			this.inputField.setText("");
	}
	
	/**
	 * Muuntaan tekstikenttn sytetyn funktion haluttuun muotoon, ja vlitt sen
	 * pohjelmalle. Jos tm ei onnistu, vlitt virheilmoituksen. Tm metodi
	 * suoritetaan, kun kyttj painaa enteri syttkentss tai klikkaa Piirr-nappia.
	 * @param e		tapahtunut ActionEvent
	 */
	public void actionPerformed(ActionEvent e) {
		try {
			String s = this.inputField.getText();
			if( s.length() > 0 ) {		// jos todella sytetty jotakin
				// listn funktio piirrettvksi
				this.parent.addFunction(new FuncXtoY(s));
				// pivitetn historiaa
				if( this.history.size() >= MAX_HISTORY_LENGTH )
					this.history.remove(0);
				this.history.add(s);
				this.currentHistoryIndex = this.history.listIterator(this.history.size());
				// tyhjennetn syttkentt
				this.inputField.setText("");
			}
		} catch( UndefinedSymbolException exception ) {		// mrittelemtn symboli
			this.parent.illegalInput("Mrittelemtn symboli: " + exception.getMessage());
		} catch( NotDefinedOperationException exception ) {	// tuntematon operaatio
			this.parent.illegalInput("Tuntematon operaatio: " + exception.getMessage());
		} catch( NumberFormatException exception ) {		// tunnistamaton luku
			this.parent.illegalInput("Virheellinen lukuformaatti: " + exception.getMessage());
		} catch( ParseException exception ) {				// jokin muu parseointipoikkeus
			this.parent.illegalInput(exception.getMessage());
		}
	}
}


